<?php

class DispletRetsIdxUsersController extends DispletRetsIdxPlugin {
	public static function get_assigned_agent_id( $user_id ) {
		if ( !empty( $user_id ) ) {
			return get_user_meta( $user_id, 'displet_agent_id', true );
		}
	}

	public static function get_assigned_lender_id( $user_id ) {
		if ( !empty( $user_id ) ) {
			return get_user_meta( $user_id, 'displet_lender_id', true );
		}
	}

	public static function is_blacklisted_email( $email ) {
		if ( !empty( self::$_options['blacklisted_emails'] ) ) {
			$blacklisted_emails = array_map( 'trim', explode( "\n", strtolower( self::$_options['blacklisted_emails'] ) ) );
			if ( !empty( $blacklisted_emails ) ) {
				$lowercase_email = strtolower( $email );
				foreach ( $blacklisted_emails as $blacklisted_email ) {
					$pattern = '/^' . str_replace( '%%', '.*', $blacklisted_email ) . '$/';
					preg_match( $pattern, $lowercase_email, $matches );
					if ( !empty( $matches ) ) {
						return true;
					}
				}
			}
		}
		return false;
	}

	public static function is_blacklisted_name( $name ) {
		if ( !empty( self::$_options['blacklisted_names'] ) ) {
			$blacklisted_names = array_map( 'trim', explode( "\n", strtolower( self::$_options['blacklisted_names'] ) ) );
			if ( !empty( $blacklisted_names ) ) {
				$lowercase_name = strtolower( $name );
				foreach ( $blacklisted_names as $blacklisted_name ) {
					$pattern = '/(?:^|\s)' . str_replace( '%%', '.*', $blacklisted_name ) . '(?:$|\s)/';
					preg_match( $pattern, $lowercase_name, $matches );
					if ( !empty( $matches ) ) {
						return true;
					}
				}
			}
		}
		return false;
	}

	public static function maybe_do_austin_home_value_new_user_jobs( $user_id, $address, $address_time ) {
		if ( !empty( $address ) && DispletRetsIdxUtilities::is_unregistered_address_theme() ) {
			if ( !empty( $user_id ) ) {
				update_user_meta( $user_id, 'displet_user_submitted_address', $address );
			}
			self::remove_partial_match_row( $address );
			wp_unschedule_event( $address_time, 'displetretsidx_send_unregistered_address_notification', array( $address ) );
		}
	}

	private static function remove_partial_match_row( $address ) {
		if ( !empty( $address ) ) {
			global $wpdb;
			$table_name = $wpdb->prefix . 'displet_partial_addresses';
			$where = array(
				'address' => $address,
			);
			$where_format = '%s';
			$wpdb->delete( $table_name, $where, $where_format );
		}
	}

	public static function submit_registration_ajax() {
		check_ajax_referer( 'displet_register_user_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_user_registration_request' ) {
			if ( !empty( $_POST['name'] ) && !empty( $_POST['email'] ) && ( empty( self::$_options['require_phone'] ) || !empty( $_POST['phone'] ) ) && ( empty( self::$_options['require_working_with_realtor'] ) || empty( self::$_options['include_working_with_realtor'] ) || !empty( $_POST['realtor'] ) ) ) {
				if ( is_email( $_POST['email'] ) ) {
					$is_blacklisted_email = self::is_blacklisted_email( $_POST['email'] );
					$is_blacklisted_name = self::is_blacklisted_name( $_POST['name'] );
					if ( !empty( $is_blacklisted_email ) ) {
						echo 'We\'re sorry, the email address provided has been prohibited.';
					}
					else if ( !empty( $is_blacklisted_name ) ) {
						echo 'We\'re sorry, the name provided has been prohibited.';
					}
					else {
						$result = self::create_new_re_search_user( array(
							'email' => $_POST['email'],
							'name' => $_POST['name'],
							'phone' => $_POST['phone'],
							'realtor' => $_POST['realtor'],
							'url' => $_POST['url'],
							'upstream_url' => $_POST['upstream_url'],
							'last_hash' => $_POST['last_hash'],
							'listing_agent_email' => $_POST['listing_agent_email'],
							'user_address' => $_POST['user_address'],
							'user_address_time' => $_POST['user_address_time'],
						) );
						if ( $result['response'] === 'Successful Registration' ) {
							do_action( 'displetretsidx_post_registration', $result['user_id'] );
						}
						echo $result['response'];
					}
				}
				else{
					echo 'Please enter a valid email address.';
				}
			}
			else{
				echo 'Please complete the required fields.';
			}
			die();
		}
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public static function create_new_re_search_user( $args ) {
		$args = wp_parse_args( $args, array(
			'email' => '',
			'name' => '',
			'phone' => '',
			'realtor' => '',
			'url' => '',
			'upstream_url' => '',
			'last_hash' => '',
			'listing_agent_email' => '',
			'user_address' => '',
			'user_address_time' => '',
			'user_registered' => '',
			'send_emails' => true,
			'login_user' => true,
			'use_cron' => true,
		 ) );
		extract( $args );
		if ( !empty( $email ) ) {
			$password = wp_generate_password( 12, false );
			//$user_id = wp_create_user( $email, $password, $email );
			$user_id = wp_insert_user( array(
				'user_login' => $email,
				'user_email' => $email,
				'user_pass' => $password,
				'role' => 'displet_user',
			 ) );
			if ( is_wp_error( $user_id ) ) {
				if ( is_multisite() && $user_id->get_error_code() == 'existing_user_login' ) {
					$user = get_user_by( 'email', $email );
					if ( !empty( $user ) ) {
						$site = get_current_site();
						$blogs = get_blogs_of_user( $user->ID );
						if ( !DispletRetsIdxUtilities::in_array_of_objects( $site->id, $blogs, 'userblog_id' ) ) {
							$success = add_user_to_blog( $site->id, $user->ID, 'displet_user' );
							if ( !empty( $success ) && !is_wp_error( $success ) ) {
								$user_id = $user->ID;
								$user_info = get_userdata( $user_id );
								if ( $login_user ) {
									wp_set_auth_cookie( $user_id );
								}
							}
						}
					}
				}
				if ( is_wp_error( $user_id ) ) {
					return array(
						'response' => $user_id->get_error_message(),
					);
				}
			}
			else{
				$user_info = get_userdata( $user_id );
				if ( $login_user ) {
					$creds = array(
						'user_login' => $user_info->user_login,
						'user_password' => $password,
						'remember' => true
					 );
					$user = wp_signon( $creds, false );
				}
			}
			$names = self::get_first_and_last_name( $name );
			$update_user_args = array(
				'ID' => $user_id,
				'role' => 'displet_user',
				'display_name' => ucwords( $name ),
			);
			if ( !empty( $user_registered ) ) {
				$time = strtotime( $user_registered );
				if ( !empty( $time ) ) {
					$update_user_args['user_registered'] = get_gmt_from_date( date_i18n( 'Y-m-d H:i:s', $time ) );
				}
			}
			wp_update_user( $update_user_args );
			$assigned_agent_id = self::get_new_duty_agent_id( $listing_agent_email );
			$assigned_lender_id = self::get_new_duty_lender_id();
			if ( $send_emails ) {
				DispletRetsIdxEmailController::send_new_user_registration( $email, $name, $phone, $realtor, $url, $user_info->user_login, $password, $assigned_agent_id, $assigned_lender_id );
			}
			$cron_job_args = array(
				'assigned_agent_id' => $assigned_agent_id,
				'assigned_lender_id' => $assigned_lender_id,
				'email' => $email,
				'first_name' => $names['first_name'],
				'last_hash' => $last_hash,
				'last_name' => $names['last_name'],
				'name' => $name,
				'phone' => $phone,
				'realtor' => $realtor,
				'upstream_url' => $upstream_url,
				'url' => $url,
				'user_address' => urldecode( $user_address ),
				'user_address_time' => $user_address_time,
				'user_id' => $user_id,
			 );
			if ( $use_cron ) {
				wp_schedule_single_event( time(), 'displetretsidx_new_user_cron_jobs', array( $cron_job_args ) );
			}
			else{
				DispletRetsIdxUsersController::new_user_cron_jobs( $cron_job_args );
			}
			return array(
				'response' => 'Successful Registration',
				'user_id' => $user_id,
			);
		}
	}

	public static function new_user_cron_jobs( $args ) {
		extract( $args );
		update_user_meta( $user_id, 'displet_lead', true );
		update_user_meta( $user_id, 'displet_agent_id', $assigned_agent_id );
		update_user_meta( $user_id, 'displet_lender_id', $assigned_lender_id );
		update_user_meta( $user_id, 'first_name', $first_name );
		update_user_meta( $user_id, 'last_name', $last_name );
		update_user_meta( $user_id, 'nickname', ucwords( $name ) );
		update_user_meta( $user_id, 'displet_phone', $phone );
		update_user_meta( $user_id, 'displet_realtor', $realtor );
		update_user_meta( $user_id, 'displet_registration_url', $url );
		update_user_meta( $user_id, 'displet_registration_upstream_url', $upstream_url );
		update_user_meta( $user_id, 'displet_last_hash', $last_hash );
		self::maybe_do_austin_home_value_new_user_jobs( $user_id, $user_address, $user_address_time );
		$zap_user = array(
			'came_from' => $upstream_url,
			'email' => $email,
			'first_name' => $first_name,
			'last_name' => $last_name,
			'name' => $name,
			'phone' => $phone,
			'registered_at' => $url,
			'user_profile_url' => admin_url( 'admin.php?page=displet-lead-manager&user_id=' . $user_id ),
			'website' => home_url(),
		 );
		self::send_to_zapier( $zap_user, 'registration' );
		$api_user_id = DispletRetsIdxUsersApiController::create_user( $email, $first_name, $last_name );
		if ( !empty( $api_user_id ) ) {
			update_user_meta( $user_id, 'displet_api_user_id', $api_user_id );
			$property_id = self::get_property_id_from_url( $url );
			DispletRetsIdxUsersApiController::update_user_property_views( $api_user_id, $property_id );
		}
	}

	public static function login_user_ajax() {
		check_ajax_referer( 'displet_login_user_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displetretsidx_user_signon' ) {
			if ( !empty( $_POST['email'] ) && ( !empty( $_POST['password'] ) || !empty( self::$_options['login_without_password'] ) ) ) {
				$logged_in = false;
				if ( !empty( self::$_options['login_without_password'] ) ) {
					$user = get_user_by( 'login', $_POST['email'] );
					if ( !empty( $user->roles ) && is_array( $user->roles ) && in_array( 'displet_user', $user->roles ) ) {
						$logged_in = true;
						wp_set_auth_cookie( $user->ID );
					}
					else{
						echo 'We were not able to locate this user. Please check your entry and try again.';
					}
				}
				else{
					$creds = array();
					$creds['user_login'] = $_POST['email'];
					$creds['user_password'] = $_POST['password'];
					$creds['remember'] = true;
					$user = wp_signon( $creds, false );
					if ( is_wp_error( $user ) ) {
						echo $user->get_error_message();
					}
					else if ( !empty( $user ) ) {
						$logged_in = true;
					}
				}
				if ( $logged_in ) {
					echo 'Successful Login';
				}
			}
			else {
				echo 'Please complete the required fields.';
			}
			die();
		}

		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public static function login_existing_facebook_user_ajax() {
		check_ajax_referer( 'displet_check_user_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_user_check_request' && !empty( $_POST['email'] ) && !empty( $_POST['token'] ) ) {
			$user_id = self::get_user_id_by_email( $_POST['email'] );
			if ( !empty( $user_id ) ) {
				$json_array = wp_remote_post( 'https://graph.facebook.com/oauth/access_token?client_id=' . self::$_options['facebook_app_id'] . '&client_secret=' . self::$_options['facebook_app_secret'] . '&grant_type=client_credentials' );
				if ( !empty( $json_array['body'] ) ) {
					$response = wp_remote_get( 'https://graph.facebook.com/debug_token?input_token=' . $_POST['token'] . '&' . $json_array['body'] );
					if ( !empty( $response['body'] ) ) {
						$result = json_decode( $response['body'] );
						if ( !empty( $result->data ) && !empty( $result->data->is_valid ) && !empty( $result->data->app_id ) && $result->data->app_id === self::$_options['facebook_app_id'] ) {
							wp_set_auth_cookie( $user_id, true );
							echo 'User Exists and Logged In';
						}
					}
				}
			}
			else {
				echo 'There is no email account associated with this user.';
			}
			die();
		}
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public static function get_user_id_by_email( $email ) {
		$user_id = username_exists( $email );
		if ( empty( $user_id ) ) {
			$user = get_user_by( 'email', $email );
			if ( empty( $user ) ) {
				return false;
			}
			else{
				$user_id = $user->ID;
			}
		}
		return $user_id;
	}

	public static function check_login_ajax() {
		check_ajax_referer( 'displet_check_login_nonce' );
		if ( is_user_logged_in() ) {
			echo 'Successful Login';
		}
		else{
			echo 'User Not Logged In';
		}
		die();
	}

	public static function get_new_duty_agent_id( $listing_agent_email, $rotate_duty_agent = true ) {
		if ( !empty( self::$_options['duty_agent_use_listing_agent_email'] ) && !empty( $listing_agent_email ) ) {
			$listing_agent_user = get_user_by( 'email', $listing_agent_email );
		}
		if ( !empty( $listing_agent_user ) ) {
			return $listing_agent_user->ID;
		}
		else{
			$duty_agent_array_position = get_option( 'displet_rets_idx_duty_agent_array_position' );
			if ( !empty( self::$_options['duty_agent'] ) ) {
				$duty_agents = array();
				foreach ( self::$_options['duty_agent'] as $agent_id => $value ) {
					if ( !empty( $value ) ) {
						$duty_agents[] = $agent_id;
					}
				}
				if ( !empty( $duty_agents ) ) {
					if ( !empty( $duty_agent_array_position ) && intval( $duty_agent_array_position ) < count( $duty_agents ) ) {
						$i = intval( $duty_agent_array_position );
					}
					else{
						$i = 0;
					}
					if ( $rotate_duty_agent ) {
						$duty_agent_array_position = $i;
						$duty_agent_array_position++;
						update_option( 'displet_rets_idx_duty_agent_array_position', $duty_agent_array_position );
					}
					return $duty_agents[$i];
				}
			}
		}
	}

	public static function get_new_duty_lender_id() {
		$duty_lender_array_position = get_option( 'displet_rets_idx_duty_lender_array_position' );
		if ( !empty( self::$_options['duty_lender'] ) ) {
			$duty_lenders = array();
			foreach ( self::$_options['duty_lender'] as $lender_id => $value ) {
				if ( !empty( $value ) ) {
					$duty_lenders[] = $lender_id;
				}
			}
			if ( !empty( $duty_lenders ) ) {
				if ( !empty( $duty_lender_array_position ) && intval( $duty_lender_array_position ) < count( $duty_lenders ) ) {
					$i = intval( $duty_lender_array_position );
				}
				else{
					$i = 0;
				}
				$duty_lender_array_position = $i;
				$duty_lender_array_position++;
				update_option( 'displet_rets_idx_duty_lender_array_position', $duty_lender_array_position );
				return $duty_lenders[$i];
			}
		}
	}

	public static function update_last_login( $user_login, $user ) {
		if ( user_can( $user->ID, 'displet_save_properties' ) && !user_can( $user->ID, 'manage_options' ) && !user_can( $user->ID, 'displet_view_leads' ) ) {
			$login_timestamps = get_user_meta( $user->ID, 'displet_logins', true );
			if ( empty( $login_timestamps ) || !is_array( $login_timestamps ) ) {
				$login_timestamps = array();
			}
			$last_login = current_time( 'timestamp' );
			$login_timestamps[] = $last_login;
			update_user_meta( $user->ID, 'displet_last_login', $last_login );
			update_user_meta( $user->ID, 'displet_logins', $login_timestamps );
			update_user_meta( $user->ID, 'displet_logins_count', count( $login_timestamps ) );
		}
	}

	public static function update_viewed_searches() {
		check_ajax_referer( 'displet_update_searches_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_update_searches' && !empty( $_POST['last_hash'] ) ) {
			$user_id = get_current_user_id();
			if ( $user_id != 0 && current_user_can( 'displet_save_searches' ) && !current_user_can( 'manage_options' ) ) {
				$user_searches = get_user_meta( $user_id, 'displet_user_hashes', true );
				$user_searches[] = $_POST['last_hash'];
				update_user_meta( $user_id, 'displet_user_hashes', $user_searches );
				update_user_meta( $user_id, 'displet_user_hashes_count', count( $user_searches ) );
			}
		}
		die();
	}

	public static function update_viewed_properties( $url = '', $address = '', $price = '', $zip = '', $sq_ft = '', $property_id = false ) {
		$user_id = get_current_user_id();
		if ( $user_id != 0 && current_user_can( 'displet_save_properties' ) && !current_user_can( 'manage_options' ) ) {
			$api_user_id = get_user_meta( $user_id, 'displet_api_user_id', true );
			$user_properties = get_user_meta( $user_id, 'displet_user_properties', true );
			$user_properties[] = array(
				'url' => $url,
				'address' => $address,
				'price' => $price,
				'zip' => $zip,
				'sq_ft' => $sq_ft
			 );
			update_user_meta( $user_id, 'displet_user_properties', $user_properties );
			if ( !empty( $api_user_id ) ) {
				DispletRetsIdxUsersApiController::update_user_property_views( $api_user_id, $property_id );
			}
			self::update_property_view_stats( $user_id, $user_properties );
		}
	}

	public static function update_property_view_stats( $user_id, $user_properties ) {
		$property_stats = DispletRetsIdxUsersController::get_property_view_stats( $user_properties );
		if ( !empty( $property_stats['price_average'] ) ) {
			update_user_meta( $user_id, 'displet_mean_price', $property_stats['price_average'] );
		}
		if ( !empty( $property_stats['square_feet_average'] ) ) {
			update_user_meta( $user_id, 'displet_mean_square_feet', $property_stats['square_feet_average'] );
		}
		if ( !empty( $property_stats['zip_mode'] ) ) {
			update_user_meta( $user_id, 'displet_zip_mode', $property_stats['zip_mode'] );
		}
	}

	public static function delete_users() {
		check_ajax_referer( 'displet_delete_users_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_delete_users_request' ) {
			if ( !empty( $_POST['displet_users'] ) ) {
				$error_occurred = false;
				foreach ( $_POST['displet_users'] as $user ) {
					$deleted[] = wp_delete_user( $user );
				}
				foreach ( $deleted as $was_deleted ) {
					if ( !$was_deleted ) {
						$error_occurred = true;
					}
				}
				if ( $error_occurred ) {
					echo 'There was an error processing your request.';
				}
				else{
					echo 'Succesful Deletion';
				}
			}
			die();
		}
	}

	public static function reassign_users() {
		check_ajax_referer( 'displet_reassign_users_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_reassign_users_request' ) {
			if ( !empty( $_POST['displet_users'] ) ) {
				if ( !empty( $_POST['displet_agent'] ) || !empty( $_POST['displet_lender'] ) ) {
					foreach ( $_POST['displet_users'] as $user ) {
						if ( !empty( $_POST['displet_agent'] ) ) {
							update_user_meta( $user, 'displet_agent_id', $_POST['displet_agent'] );
						}
						if ( !empty( $_POST['displet_lender'] ) ) {
							update_user_meta( $user, 'displet_lender_id', $_POST['displet_lender'] );
						}
						DispletRetsIdxEmailController::send_assigned_lead_message( $user, $_POST['displet_agent'], $_POST['displet_lender'] );
					}
					echo 'Succesful Assignation';
				}
				else{
					echo 'No agent or lender has been selected.';
				}
			}
			die();
		}
	}

	public static function save_property() {
		check_ajax_referer( 'displet_save_property_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_save_property_request' ) {
			if ( current_user_can( 'displet_save_properties' ) && !current_user_can( 'manage_options' ) ) {
				if ( !empty( $_POST['type'] ) ) {
					$user = wp_get_current_user();
					if ( !empty( $user->ID ) ) {
						$base = get_permalink( self::$_options['property_details_page_id'] );
						$url = str_replace( $base, '', $_POST['url'] );
						$saved_properties = get_user_meta( $user->ID, 'displet_saved_properties', true );
						$saved_properties[] = array(
							'sysid' => $_POST['sysid'],
							'url' => $url,
							'type' => $_POST['type'],
							'rating' => $_POST['rating'],
							'message' => $_POST['message'],
						);
						update_user_meta( $user->ID, 'displet_saved_properties', $saved_properties );
						update_user_meta( $user->ID, 'displet_saved_properties_count', count( $saved_properties ) );
						new DispletRetsIdxEmail( 'saved_property', array(
							'agent_id' => self::get_assigned_agent_id( $user->ID ),
							'lender_id' => self::get_assigned_lender_id( $user->ID ),
							'property_address' => $_POST['address'],
							'property_url' => $_POST['url'],
							'user_email' => $user->user_email,
							'user_message' => $_POST['message'],
							'user_name' => $user->display_name,
							'user_phone' => $user->displet_phone,
						) );
						echo 'This property has been saved.';
					}
					else{
						echo 'You are not logged in. Please sign in or create an account and try again.';
					}
				}
				else{
					echo 'Please select Save as favorite, Save as possibility, or Just save notes.';
				}
			}
			else{
				echo 'The current user does not have the capability to save properties.';
			}
			die();
		}
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public static function delete_saved_properties() {
		check_ajax_referer( 'displet_delete_properties_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_delete_properties_request' ) {
			if ( !empty( $_POST['displet_properties'] ) ) {
				$user_id = get_current_user_id();
				$saved_properties = get_user_meta( $user_id, 'displet_saved_properties' );
				$delete_properties = $_POST['displet_properties'];
				foreach ( $delete_properties as $property_id ) {
					unset( $saved_properties[0][$property_id] );
				}
				$saved_properties[0] = array_values( $saved_properties[0] );
				update_user_meta( $user_id, 'displet_saved_properties', $saved_properties[0] );
				echo 'Succesful Deletion';
			}
			else{
				echo 'No property was selected for deletion.';
			}
			die();
		}
		else {
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public static function save_search() {
		check_ajax_referer( 'displet_save_search_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_save_search_request' ) {
			if ( !empty( $_POST['hash'] ) ) {
				if ( current_user_can( 'displet_view_leads' ) ) {
					$user = get_userdata( $_POST['client_id'] );
				}
				else{
					$user = wp_get_current_user();
				}
				if ( !empty( $user->ID ) ) {
					if ( current_user_can( 'displet_save_searches' ) ) {
						if ( !empty( $_POST['search_name'] ) ) {
							$saved_searches = get_user_meta( $user->ID, 'displet_saved_searches', true );
							$name_exists = false;
							if ( !empty( $saved_searches ) && is_array( $saved_searches ) ) {
								foreach ( $saved_searches as $saved_search ) {
									if ( $saved_search['name'] == $_POST['search_name'] ) {
										$name_exists = true;
									}
								}
							}
							if ( empty( $name_exists ) ) {
								$this_search = array(
									'hash' => $_POST['hash'],
									'name' => $_POST['search_name']
								 );
								$api_user_id = get_user_meta( $user->ID, 'displet_api_user_id', true );
								if ( !empty( $api_user_id ) ) {
									$saved_search_criteria = self::get_search_criteria_from_hash( $_POST['hash'] );
									$saved_search_id = DispletRetsIdxUsersApiController::create_saved_search( $api_user_id, $_POST['search_name'], $saved_search_criteria );
									if ( isset( $saved_search_id ) ) {
										$this_search['api_id'] = $saved_search_id;
									}
								}
								$saved_searches[] = $this_search;
								update_user_meta( $user->ID, 'displet_saved_searches', $saved_searches );
								new DispletRetsIdxEmail( 'saved_search', array(
									'agent_id' => self::get_assigned_agent_id( $user->ID ),
									'lender_id' => self::get_assigned_lender_id( $user->ID ),
									'search_url' => trailingslashit( get_permalink( self::$_options['search_results_page_id'] ) ) . $_POST['hash'],
									'user_email' => $user->user_email,
									'user_name' => $user->display_name,
									'user_phone' => $user->displet_phone,
								) );
								echo 'Saved Search';
								die();
							}
							else {
								echo 'The search "' . $_POST['search_name'] . '" already exists. Please select a different name.';
							}
						}
						else {
							echo 'Please name the search.';
						}
					}
					else{
						echo 'The current user does not have the capability to save properties.';
					}
				}
				else{
					echo 'Not Logged In';
				}
			}
			else{
				echo 'Please perform a search to save.';
			}
			die();
		}

		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public static function delete_saved_searches() {
		check_ajax_referer( 'displet_delete_searches_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_delete_searches_request' ) {
			if ( !empty( $_POST['api_search_ids'] ) && is_array( $_POST['api_search_ids'] ) ) {
				$user_id = get_current_user_id();
				$saved_searches = get_user_meta( $user_id, 'displet_saved_searches', true );
				$api_user_id = get_user_meta( $user_id, 'displet_api_user_id', true );
				foreach ( $saved_searches as $key => $saved_search ) {
					if ( !empty( $saved_search['api_id'] ) && in_array( $saved_search['api_id'], $_POST['api_search_ids'] ) ) {
						DispletRetsIdxUsersApiController::delete_saved_search( $api_user_id, $saved_search['api_id'] );
						unset( $saved_searches[ $key ] );
					}
				}
				update_user_meta( $user_id, 'displet_saved_searches', array_values( $saved_searches ) );
				echo 'Succesful Deletion';
			}
			else{
				echo 'No property was selected for deletion.';
			}
			die();
		}
		else {
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public static function get_clients_ajax() {
		check_ajax_referer( 'displet_get_clients_nonce' );
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'displet_get_clients_request' ) {
			$args = array(
				'orderby' => 'display_name',
			);
			if ( current_user_can( 'manage_options' ) ) {
				$clients = self::get_re_search_users( false, $args );
			}
			else{
				$user_id = get_current_user_id();
				if ( !empty( $user_id ) ) {
					$clients = self::get_re_search_users( $user_id, $args );
				}
			}
			if ( !empty( $clients ) && is_array( $clients ) ) {
				$output = array();
				foreach ( $clients as $client ) {
					$output[] = array(
						'name' => $client->display_name,
						'id' => $client->ID
					 );
				}
				echo json_encode( $output );
			}
			else{
				echo 'No matching users.';
			}
		}
		else{
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public static function set_referral_cookie() {
		if ( empty( $_COOKIE['displetretsidx_referring_site'] ) ) {
        	setcookie( 'displetretsidx_referring_site', $_SERVER['HTTP_REFERER'], 0, '/' );
		}
	}

	private function get_search_criteria_from_hash( $hash ) {
		$trimmed_saved_search_criteria = rtrim( ltrim( $hash, '#' ), '/' );
		$saved_search_criteria = str_replace( '/', '&', $trimmed_saved_search_criteria );
		return $saved_search_criteria;
	}

	public static function get_first_and_last_name( $name ) {
		$names = array();
		preg_match( '/[^\s]+/', $name, $first_name_match );
		if ( !empty( $first_name_match ) ) {
			$names['first_name'] = $first_name_match[0];
		}
		else{
			$names['first_name'] = $name;
		}
		$names['last_name'] = trim( str_replace( $names['first_name'], '', $name ) );
		if ( !empty( $names['first_name'] ) ) {
			$names['first_name'] = ucwords( $names['first_name'] );
		}
		if ( !empty( $names['last_name'] ) ) {
			$names['last_name'] = ucwords( $names['last_name'] );
		}
		return $names;
	}

	public static function send_users_saved_searches_to_api( $user_id, $api_user_id ) {
		if ( !empty( $user_id ) && !empty( $api_user_id ) ) {
			$saved_searches = get_user_meta( $user_id, 'displet_saved_searches', true );
			if ( !empty( $saved_searches ) ) {
				$new_saved_searches = array();
				foreach ( $saved_searches as $saved_search ) {
					if ( is_array( $saved_search ) ) {
						$hash = $saved_search['hash'];
						$name = $saved_search['name'];
					}
					else{
						$hash = $saved_search;
						$name = false;
					}
					if ( empty( $name ) ) {
						$name = wp_generate_password( 12, false );
					}
					$criteria = self::get_search_criteria_from_hash( $hash );
					$saved_search_id = DispletRetsIdxUsersApiController::create_saved_search( $api_user_id, $name, $criteria );
					$new_saved_searches[] = array(
						'hash' => $hash,
						'name' => $name,
						'api_id' => $saved_search_id
					 );
				}
				update_user_meta( $user_id, 'displet_saved_searches', $new_saved_searches );
			}
		}
	}

	public static function get_property_id_from_url( $url ) {
		if ( !empty( $url ) ) {
			$pdp_url = get_permalink( self::$_options['property_details_page_id'] );
			$clean_url = str_replace( $pdp_url, '', $url );
			preg_match( '#[^/]+/[^/]+/[^/]+/( [^/]+ )#', $clean_url, $matches );
			if ( !empty( $matches ) ) {
				return $matches[1];
			}
		}
	}

	public static function add_custom_user_role_for_leads() {
		remove_role( 'displet_user' );
		add_role( 'displet_user', 'Real Estate Search User', array(
			'displet_save_properties' => true,
			'displet_save_searches' => true,
			'read' => true,
		) );
	}

	public static function add_custom_user_role_for_agents() {
		remove_role( 'displet_agent' );
		add_role( 'displet_agent', 'Displet RE Agent', array(
			'displet_save_properties' => true,
			'displet_save_searches' => true,
			'displet_view_leads' => true,
			'edit_users' => true,
			'read' => true,
		) );
	}

	public static function add_custom_user_role_for_lenders() {
		remove_role( 'displet_lender' );
		add_role( 'displet_lender', 'Displet RE Lender', array(
			'displet_view_leads' => true,
			'edit_users' => true,
			'read' => true,
		) );
	}

	public static function give_admins_custom_capabilities( $capabilities, $capability, $args ) {
		if ( !empty( $capabilities['manage_options'] ) && ( $args[0] === 'displet_view_leads' || $args[0] === 'displet_save_searches' ) ) {
			$capabilities[ $capability[0] ] = true;
		}
		return $capabilities;
	}

	public static function send_to_zapier( $user_data, $zap ) {
		if ( !empty( self::$_options['zapier_' . $zap . '_url'] ) ) {
			$request = array(
				'user-agent' => 'Displet',
				'body' => $user_data,
				'referer' => home_url()
			 );
			$urls = array_map( 'trim', explode( "\n", self::$_options['zapier_' . $zap . '_url'] ) );
			foreach ( $urls as $url ) {
				$post_url = trim( $url );
				if ( !empty( $post_url ) ) {
					wp_remote_post( $post_url, $request );
				}
			}
		}
	}

	public static function add_to_agent_page( $user ) {
		if ( user_can( $user->ID, 'displet_view_leads' ) ) {

			?>
				<h3>Displet Settings</h3>
				<table class="form-table">
					<tr>
						<th><label for="displet_email_signature">Email Signature</label></th>
						<td>
							<textarea name="displet_email_signature" id="displet_email_signature" rows="5" cols="30"><?php echo esc_attr( get_the_author_meta( 'displet_email_signature', $user->ID ) ); ?></textarea><br />
							<span class="description">Displayed at the bottom of each outbound email.</span>
						</td>
					</tr>
				</table>
			<?php
		}
	}

	public static function save_from_agent_page( $user_id ) {
		$saved = false;
		if ( user_can( $user_id, 'displet_view_leads' ) && current_user_can( 'edit_user', $user_id ) ) {
			update_user_meta( $user_id, 'displet_email_signature', $_POST['displet_email_signature'] );
			$saved = true;
		}
		return true;
	}

	public static function add_to_user_page( $user ) {
		if ( user_can( $user->ID, 'displet_save_properties' ) ) {
			?>
				<table class="form-table">
					<tr>
						<th><label for="displet_phone">Phone</label></th>
						<td>
							<input type="text" name="displet_phone" id="displet_phone" value="<?php echo esc_attr( get_the_author_meta( 'displet_phone', $user->ID ) ); ?>" class="regular-text"/><br />
						</td>
					</tr>
				</table>
			<?php
		}
	}

	public static function save_from_user_page( $user_id ) {
		if ( user_can( $user_id, 'displet_save_properties' ) && current_user_can( 'edit_user', $user_id ) ) {
			if ( isset( $_POST['displet_phone'] ) ) {
				update_user_meta( $user_id, 'displet_phone', $_POST['displet_phone'] );
			}
			$api_user_id = get_user_meta( $user_id, 'displet_api_user_id', true );
			if ( !empty( $api_user_id ) ) {
				$user_array = array();
				if ( !empty( $_POST['first_name'] ) ) {
					$user_array['first_name'] = $_POST['first_name'];
				}
				if ( !empty( $_POST['last_name'] ) ) {
					$user_array['last_name'] = $_POST['last_name'];
				}
				if ( !empty( $_POST['email'] ) ) {
					$user_array['email'] = $_POST['email'];
				}
				DispletRetsIdxUsersApiController::update_user( $api_user_id, $user_array );
			}
		}
	}

	public static function get_property_view_stats( $properties ) {
		$stats = array(
			'price_average' => '',
			'square_feet_average' => '',
			'zip_mode' => ''
		 );
		if ( !empty( $properties ) && is_array( $properties ) ) {
			$price_total = $price_count = $sq_ft_total = $sq_ft_count = 0;
			$properties_zips = array();
			foreach ( $properties as $property ) {
				if ( !empty( $property['price'] ) && intval( $property['price'] ) > 0 ) {
					$price_total += intval( $property['price'] );
					$price_count++;
				}
				if ( !empty( $property['sq_ft'] ) && intval( $property['sq_ft'] ) > 0 ) {
					$sq_ft_total += intval( $property['sq_ft'] );
					$sq_ft_count++;
				}
				if ( !empty( $property['zip'] ) ) {
					$properties_zips[] = $property['zip'];
				}
			}
			if ( $price_count > 0 ) {
				$stats['price_average'] = intval( $price_total / $price_count );
			}
			if ( $sq_ft_count > 0 ) {
				$stats['square_feet_average'] = intval( $sq_ft_total / $sq_ft_count );
			}
			if ( !empty( $properties_zips ) ) {
				$values = array_count_values( $properties_zips );
				$stats['zip_mode'] = array_search( max( $values ), $values );
			}
		}
		return $stats;
	}

	public static function update_property_inquiries( $user_id, $details ) {
		if ( !empty( $user_id ) ) {
			$inquiries = get_user_meta( $user_id, 'displet_property_inquiries', true );
			if ( empty( $inquiries ) ) {
				$inquiries = array();
			}
			$inquiries[] = $details;
			update_user_meta( $user_id, 'displet_property_inquiries', $inquiries );
			update_user_meta( $user_id, 'displet_property_inquiries_count', count( $inquiries ) );
		}
	}

	public static function get_re_search_users( $agent_id = false, $args = array() ) {
		if ( !empty( $agent_id ) ) {
			$args['meta_key'] = 'displet_agent_id';
			$args['meta_value'] = $agent_id;
		}
		else{
			$args['role'] = self::$_roles['lead'];
			/*
			$args['meta_key'] = 'displet_lead';
			$args['meta_value'] = true;
			*/
		}
		return get_users( $args );
	}

	public static function filter_lead_manager_query_vars() {
		parse_str( $_SERVER['QUERY_STRING'], $url_query_vars );
		$original_count = count( $url_query_vars );
		$url_query_vars = array_filter( $url_query_vars );
		if ( $original_count > count( $url_query_vars ) ) {
			$url = admin_url( 'admin.php?' . http_build_query( $url_query_vars ) );
			wp_safe_redirect( $url, 301 );
		}
	}

	public function import_users_from_csv( $csv_path, $send_emails ) {
		if ( !empty( $csv_path ) ) {
			$handle = fopen( $csv_path, 'r' );
			if ( !empty( $handle ) ) {
				$keys = array_map( 'strtolower', fgetcsv( $handle, 0 ) );
				while ( !feof( $handle ) ) {
					$csv_array = fgetcsv( $handle, 0 );
					if ( !empty( $csv_array ) ) {
				    	$users[] = array_combine( $keys, $csv_array );
					}
				}
				wp_schedule_single_event( time(), 'displetretsidx_import_users_cron_job', array( $users, $send_emails ) );
			    fclose( $handle );
			}
		}
	}

	public static function import_users_cron_job( $users, $send_emails ) {
		if ( !empty( $users ) && is_array( $users ) ) {
			$time = time();
			foreach ( $users as $user ) {
				if ( !empty( $user['email'] ) ) {
					wp_schedule_single_event( $time, 'displetretsidx_import_single_user_cron_job', array( $user, $send_emails ) );
				}
				$time += 30;
			}
		}
	}

	public static function import_single_user_cron_job( $user, $send_emails ) {
		if ( !empty( $user['email'] ) ) {
			DispletRetsIdxUsersController::create_new_re_search_user( array(
				'email' => $user['email'],
				'name' => $user['name'],
				'phone' => $user['phone'],
				'user_registered' => $user['date'],
				'url' => 'User Import',
				'send_emails' => !empty( $send_emails ) ? true : false,
				'login_user' => false,
				'use_cron' => false
			 ) );
		}
	}
}

?>